package com.io.yutian.elementoriginlib.command;

import com.io.yutian.elementoriginlib.command.argument.Argument;
import com.io.yutian.elementoriginlib.command.argument.ArgumentValue;
import com.io.yutian.elementoriginlib.command.interfaces.Parameter;
import com.io.yutian.elementoriginlib.logger.Logger;
import org.bukkit.command.CommandSender;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CommandEntry {

    private static final Logger LOGGER = Logger.getLogger(CommandEntry.class);

    private String fullName;
    private String name;
    private String permission;
    private List<SenderRequire> senderRequires = new ArrayList<>();

    private boolean nodal;

    private List<Argument> arguments = new ArrayList<>();

    private List<CommandEntry> childrens = new ArrayList<>();

    private long depth = 0;

    private final Method method;
    private final MethodHandle methodHandle;

    private Class<?>[] parameterTypes;

    public CommandEntry(String fullName, String name, String permission, boolean nodal, List<SenderRequire> senderRequires, List<Argument> arguments, Method method) {
        this.fullName = fullName;
        this.name = name;
        this.permission = permission;
        this.nodal = nodal;
        this.senderRequires = senderRequires;
        this.arguments = arguments;
        this.method = method;
        try {
            this.methodHandle = MethodHandles.lookup().unreflect(this.method);
            this.parameterTypes = methodHandle.type().parameterArray();
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
        this.depth = fullName.chars().filter(ch -> ch == '.').count();
    }

    public CommandEntry(String fullName, String name) {
        this.fullName = fullName;
        this.name = name;
        this.method = null;
        this.methodHandle = null;
        this.parameterTypes = new Class<?>[0];
    }

    public void invoke(CommandContext commandContext) {
        if (method == null || methodHandle == null) {
            return;
        }

        Object targetInstance = commandContext.getCommandInstance();
        if (targetInstance == null) {
            LOGGER.warn("该命令的实例为空，无法执行命令: " + fullName);
            return;
        }

        try {
            Class<?>[] parameterTypes = method.getParameterTypes();
            Object[] parameters = new Object[parameterTypes.length];

            int argumentIndex = 0;

            for (int i = 0; i < parameterTypes.length; i++) {
                java.lang.reflect.Parameter parameter = method.getParameters()[i];
                if (parameter.isAnnotationPresent(Parameter.class)) {
                    String paramName = arguments.get(argumentIndex).getName();
                    ArgumentValue argumentValue = commandContext.getArgumentsValue(paramName);
                    if (argumentValue != null) {
                        parameters[i] = argumentValue.getValue();
                    } else {
                        parameters[i] = null;
                    }
                    argumentIndex++;
                } else if (parameter.getType() == CommandContext.class) {
                    parameters[i] = commandContext;
                } else {
                    parameters[i] = getDefaultForType(parameterTypes[i]);
                }
            }

            Object[] parms = new Object[parameters.length+1];
            parms[0] = targetInstance;
            System.arraycopy(parameters, 0, parms, 1, parameters.length);
            methodHandle.invokeWithArguments(parms);
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error("执行命令时出现异常: " + fullName);
        } catch (Throwable e) {
            e.printStackTrace();
            LOGGER.error("执行命令时出现异常: " + fullName);
        }
    }

    public boolean canInvoke(CommandSender sender) {
        if (senderRequires != null && senderRequires.size() > 0) {
            for (SenderRequire senderRequire : senderRequires) {
                if (senderRequire.test(sender)) {
                    return true;
                }
            }
            return false;
        }
        return true;
    }

    public boolean hasPermission(CommandSender sender) {
        if (permission != null && !permission.isEmpty()) {
            return sender.isOp() || sender.hasPermission(permission);
        }
        return true;
    }

    private Object[] convertToPrimitive(Object[] parameters) {
        Object[] convertedParams = new Object[parameters.length];
        for (int i = 0; i < parameters.length; i++) {
            Object param = parameters[i];
            Class<?> parameterType = parameterTypes[i+1];
            if (parameterType.isPrimitive()) {
                if (param instanceof Integer) {
                    convertedParams[i] = ((Integer) param).intValue();
                } else if (param instanceof Double) {
                    convertedParams[i] = ((Double) param).doubleValue();
                } else if (param instanceof Boolean) {
                    convertedParams[i] = ((Boolean) param).booleanValue();
                } else if (param instanceof Character) {
                    convertedParams[i] = ((Character) param).charValue();
                } else if (param instanceof Byte) {
                    convertedParams[i] = ((Byte) param).byteValue();
                } else if (param instanceof Short) {
                    convertedParams[i] = ((Short) param).shortValue();
                } else if (param instanceof Long) {
                    convertedParams[i] = ((Long) param).longValue();
                } else if (param instanceof Float) {
                    convertedParams[i] = ((Float) param).floatValue();
                } else {
                    convertedParams[i] = param;
                }
            }
        }
        return convertedParams;
    }

    private Object getDefaultForType(Class<?> parameterType) {
        if (parameterType.isPrimitive()) {
            if (parameterType == int.class) {
                return 0;
            } else if (parameterType == double.class) {
                return 0.0;
            } else if (parameterType == boolean.class) {
                return false;
            } else if (parameterType == char.class) {
                return '\u0000';
            } else if (parameterType == byte.class) {
                return (byte) 0;
            } else if (parameterType == short.class) {
                return (short) 0;
            } else if (parameterType == long.class) {
                return 0L;
            } else if (parameterType == float.class) {
                return 0.0f;
            }
        }
        return null;
    }

    public boolean isEmptyEntry() {
        return method == null;
    }

    public void addChild(CommandEntry child) {
        childrens.add(child);
    }

    public String getFullName() {
        return fullName;
    }

    public String getName() {
        return name;
    }

    public String getPermission() {
        return permission;
    }

    public List<SenderRequire> getSenderRequires() {
        return senderRequires;
    }

    public List<Argument> getArguments() {
        return arguments;
    }

    public boolean isNodal() {
        return nodal;
    }

    public List<CommandEntry> getChildrens() {
        return childrens;
    }

    public long getDepth() {
        return depth;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Override
    public String toString() {
        return "CommandEntry{" +
                "fullName='" + fullName + '\'' +
                ", name='" + name + '\'' +
                ", permission='" + permission + '\'' +
                ", senderRequires=" + senderRequires +
                ", nodal=" + nodal +
                ", arguments=" + arguments +
                ", childrens=" + childrens +
                ", depth=" + depth +
                ", method=" + method +
                ", methodHandle=" + methodHandle +
                ", parameterTypes=" + Arrays.toString(parameterTypes) +
                '}';
    }
}
