package com.io.yutian.elementoriginlib.command.argument;

import com.io.yutian.elementoriginlib.util.ClassUtil;
import com.io.yutian.elementoriginlib.util.StringUtil;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;
import java.util.function.Predicate;

public class ArgumentType<T> {

    private static final ArgumentType<String> STRING = new ArgumentType<>("string", (s) -> true, (s) -> s);

    private static final ArgumentType<Integer> INTEGER = new ArgumentType<>("integer", StringUtil::isInt, Integer::parseInt);

    private static final ArgumentType<Double> DOUBLE = new ArgumentType<>("double", StringUtil::isDouble, Double::parseDouble);

    private static final ArgumentType<Boolean> BOOLEAN = new ArgumentType<>("boolean", StringUtil::isBoolean, Boolean::parseBoolean);

    private static final ArgumentType<String[]> STRING_ARRAY = new ArgumentType<>("string[]", (s) -> true, (s) -> s.split(","));

    private static final ArgumentType<Integer[]> INTEGER_ARRAY = new ArgumentType<>("integer[]", (s) -> true, (s) -> {
        String[] arr = s.split(",");
        Integer[] result = new Integer[arr.length];
        for (int i = 0; i < arr.length; i++) {
            result[i] = Integer.parseInt(arr[i]);
        }
        return result;
    });
    private static final ArgumentType<Double[]> DOUBLE_ARRAY = new ArgumentType<>("double[]", (s) -> true, (s) -> {
        String[] arr = s.split(",");
        Double[] result = new Double[arr.length];
        for (int i = 0; i < arr.length; i++) {
            result[i] = Double.parseDouble(arr[i]);
        }
        return result;
    });
    private static final ArgumentType<Boolean[]> BOOLEAN_ARRAY = new ArgumentType<>("boolean[]", (s) -> true, (s) -> {
        String[] arr = s.split(",");
        Boolean[] result = new Boolean[arr.length];
        for (int i = 0; i < arr.length; i++) {
            result[i] = Boolean.parseBoolean(arr[i]);
        }
        return result;
    });

    private static final Map<Class<?>, ArgumentType<?>> ARGUMENT_TYPES = new HashMap<>();

    private final String name;

    private Predicate<String> predicate;
    private Function<String, T> function;

    public ArgumentType(String name, Predicate<String> predicate, Function<String, T> function) {
        this.name = name;
        this.predicate = predicate;
        this.function = function;
    }

    public String getName() {
        return name;
    }

    public boolean test(String string) {
        return predicate.test(string);
    }

    public T get(String t) {
        return function.apply(t);
    }

    static {
        register(String.class, STRING);
        register(Integer.class, INTEGER);
        register(Double.class, DOUBLE);
        register(Boolean.class, BOOLEAN);
        register(String[].class, STRING_ARRAY);
        register(Integer[].class, INTEGER_ARRAY);
        register(Double[].class, DOUBLE_ARRAY);
        register(Boolean[].class, BOOLEAN_ARRAY);
    }

    public static <T> void register(Class<T> clazz, ArgumentType<T> type) {
        ARGUMENT_TYPES.put(clazz, type);
    }

    public static Map<Class<?>, ArgumentType<?>> getArgumentTypes() {
        return ARGUMENT_TYPES;
    }

    public static <T> ArgumentType<T> get(Class<T> clazz) {
        if (clazz.isEnum()) {
            return new EnumArgumentType<>("enum_"+clazz.getSimpleName(), (Class<? extends Enum>)clazz);
        }
        Class<?> c = ClassUtil.PRIMITIVE_TO_WRAPPER.getOrDefault(clazz, clazz);
        return (ArgumentType<T>) ARGUMENT_TYPES.get(c);
    }

    private static class EnumArgumentType<T extends Enum<T>> extends ArgumentType<T> {

        private final Class<T> clazz;

        public EnumArgumentType(String name, Class<T> clazz) {
            super(name, (s)->{
                try {
                    Enum.valueOf(clazz, s);
                    return true;
                } catch (Exception e) {
                    return false;
                }
            }, (s)-> Enum.valueOf(clazz, s));
            this.clazz = clazz;
        }

    }

}
