package com.io.yutian.elementoriginlib.command.handler;

import com.io.yutian.elementoriginlib.command.*;
import com.io.yutian.elementoriginlib.command.argument.Argument;
import com.io.yutian.elementoriginlib.command.argument.ArgumentValue;
import com.io.yutian.elementoriginlib.lang.Lang;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;

import java.util.*;
import java.util.stream.Collectors;

public class CommandHandler implements CommandExecutor, TabCompleter {

    private final ICommandManager commandManager;

    public CommandHandler(ICommandManager commandManager) {
        this.commandManager = commandManager;
    }

    public void execute(CommandSender sender, String label, String[] args) {
        if (args.length == 0) {
            sender.sendMessage(Lang.get("command.short-arg"));
            return;
        }
        String commandName = args[0];
        Optional<CommandEntity> entityOptional = commandManager.getCommandEntities().stream()
                .filter(entity -> entity.getCommand().equalsIgnoreCase(commandName))
                .findFirst();

        if (!entityOptional.isPresent()) {
            sender.sendMessage(Lang.get("command.unknown", commandName));
            return;
        }

        CommandEntity commandEntity = entityOptional.get();
        int index = 1;
        if (commandEntity.getChildrens().size() == 1 && commandEntity.getChildrens().get(0).isNodal()) {
            index = 0;
        }
        handleCommand(commandEntity, sender, label, args, commandEntity.getChildrens(), index, new StringBuilder());
    }

    private void handleCommand(CommandEntity commandEntity, CommandSender sender, String label, String[] args, List<CommandEntry> entries, int index, StringBuilder fullCommand) {
        if (index >= args.length) {
            sender.sendMessage(Lang.get("command.short-arg"));
            return;
        }

        String currentArg = args[index];
        Optional<CommandEntry> optionalEntry = entries.stream()
                .filter(entry -> entry.getName().equalsIgnoreCase(currentArg))
                .findFirst();

        if (optionalEntry.isEmpty()) {
            sender.sendMessage(Lang.get("command.unknown-arg", index + 1, currentArg));
            return;
        }

        CommandEntry entry = optionalEntry.get();
        fullCommand.append(currentArg);

        if (!entry.canInvoke(sender)) {
            sender.sendMessage(Lang.get("command.error-sender-type"));
            return;
        }

        if (!entry.hasPermission(sender)) {
            sender.sendMessage(Lang.get("command.no-permission"));
            return;
        }

        if (!entry.getChildrens().isEmpty()) {
            fullCommand.append(" ");
            handleCommand(commandEntity, sender, label, args, entry.getChildrens(), index + 1, fullCommand);
        } else {
            if (!validateArguments(sender, entry.getArguments(), args, index + 1)) {
                return;
            }
            Map<String, ArgumentValue> parsedArguments = parseArgumentValue(sender, entry.getArguments(), args, index + 1);
            entry.invoke(new CommandContext(commandEntity.getInstance(), fullCommand.toString(), label, sender, parsedArguments));
        }
    }

    private boolean validateArguments(CommandSender sender, List<Argument> arguments, String[] args, int startIndex) {
        for (int i = 0; i < arguments.size(); i++) {
            Argument argument = arguments.get(i);
            int argIndex = startIndex + i;
            if (argIndex >= args.length) {
                if (argument.isOptional()) {
                    continue;
                }
                sender.sendMessage(Lang.get("command.short-arg"));
                return false;
            }

            String arg = args[argIndex];
            if (!argument.getArgumentsType().test(arg)) {
                sender.sendMessage(Lang.get("command.error-arg", argIndex + 1, arg));
                return false;
            }
        }
        return true;
    }

    private Map<String, ArgumentValue> parseArgumentValue(CommandSender sender, List<Argument> arguments, String[] args, int startIndex) {
        Map<String, ArgumentValue> parsedArguments = new HashMap<>();
        List<Object> methodParams = new ArrayList<>();

        for (Argument argument : arguments) {
            int argIndex = startIndex + arguments.indexOf(argument);

            if (argIndex >= args.length) {
                if (!argument.isOptional()) {
                    sender.sendMessage(Lang.get("command.short-arg", argument.getName()));
                    return null;
                }
            }

            String rawValue = null;
            if (argument.isOptional() && argIndex >= args.length) {
                rawValue = argument.getDefaultValue();
            } else {
                rawValue = args[argIndex];
            }
            if (!argument.getArgumentsType().test(rawValue)) {
                sender.sendMessage(Lang.get("command.error-arg", argIndex + 1, rawValue));
                return null;
            }
            Object parsedValue = argument.getArgumentsType().get(rawValue);
            parsedArguments.put(argument.getName(), new ArgumentValue(parsedValue));

            methodParams.add(parsedValue);
        }

        return parsedArguments;
    }

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        execute(sender, label, args);
        return true;
    }

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        if (args.length == 1) {
            return commandManager.getCommandEntities().stream()
                    .map(CommandEntity::getCommand)
                    .collect(Collectors.toList());
        }

        String commandName = args[0];
        Optional<CommandEntity> entityOptional = commandManager.getCommandEntities().stream()
                .filter(entity -> entity.getCommand().equalsIgnoreCase(commandName))
                .findFirst();

        if (!entityOptional.isPresent()) {
            return Collections.emptyList();
        }

        CommandEntity commandEntity = entityOptional.get();
        List<CommandEntry> entries = commandEntity.getChildrens();

        if (entries.isEmpty()) {
            return Collections.emptyList();
        }

        int depth = args.length;
        String currentArg = args[depth - 1];

        return getSuggestions(sender, entries, args, 0, depth - 1, currentArg);
    }


    private List<String> getSuggestions(CommandSender sender, List<CommandEntry> entries, String[] args, int start, int currentDepth, String currentArg) {
        CommandEntry commandEntry = null;

        for (int i = start; i < currentDepth; i++) {
            String arg = args[i];
            Optional<CommandEntry> optionalEntry = entries.stream()
                    .filter(entry -> entry.getName().equalsIgnoreCase(arg))
                    .findFirst();

            if (!optionalEntry.isPresent()) {
                continue;
            }

            commandEntry = optionalEntry.get();

            entries = commandEntry.getChildrens();
        }

        if (entries.size() == 0) {
            List<Argument> arguments = commandEntry.getArguments();
            int index = (int) (currentDepth - commandEntry.getDepth() - 1);
            if (index >= arguments.size()) {
                return Collections.emptyList();
            }
            if (!arguments.isEmpty()) {
                Argument argument = arguments.get(index);
                Suggest suggest = argument.getSuggest();
                if (suggest != null) {
                    return suggest.getSuggest();
                } else {
                    return Collections.singletonList(new StringBuilder().append("<").append(argument.getName()).append(">").toString());
                }
            }
        }

        return entries.stream()
                .filter(entry -> entry.getName().toLowerCase().startsWith(currentArg.toLowerCase()))
                .filter(entry -> entry.canInvoke(sender) && entry.hasPermission(sender))
                .map(CommandEntry::getName)
                .collect(Collectors.toList());
    }

}